#if defined(ANDROID) || defined(ANDROID_TWO_WORLDS)
//#define __TEST_BY_OSD__
#include <linux/types.h>
#include <linux/stat.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/sched.h>
#include <linux/export.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <linux/sched.h>
#include "imgrz_if.h"
#include "imgrz_drvif.h"
#include "fbm_drvif.h"
#include "srm_drvif.h"
#include "vdp_drvif.h"
#include "vdp_if.h"
#include "b2r_if.h"
#include "b2r_drvif.h"
#include "vdec_drvif.h"
#include "vdo_if.h"
#include "pe_if.h"
#include "ostg_if.h"
#include "eeprom_if.h"
#include "drv_tdtv_drvif.h"
#include "mtgfx.h"
#include "gfx_drvif.h"
#include "gfx_if.h"
#include "osd_drvif.h"
#include "mt53_simple.h"
#include "mt53_accel.h"

#define INIT_LIST_HEAD(X) {.prev = &X, .next = &X}

#define MAX_CHANNEL_NS    2
#define MAX_IMGRZINFO_NS  2
#define VDP_MAIN          0
#define VDP_SUB           1

typedef BOOL (*ConsumeListHandle)(SIMPLE_BUFFER_T *prBuffer);
typedef struct CONSUME_BUFFER_T CONSUME_BUFFER_T;

struct CONSUME_BUFFER_T
{
    struct list_head link;
	SIMPLE_BUFFER_T rBuffer;
};

typedef struct
{
    HANDLE_T hConvertThreadOne;
    HANDLE_T hEventMsgQ;
    HANDLE_T hImgrzSema;   
    BOOL  fgConvertThreadOneRunning;
} SWVDEC_VDP_CTX_T;

typedef struct
{
    UINT32  u4ImgrzID;
    UINT32  u4ChannelID;
    UINT32  u4ScrAddr;
    UINT32  u4SrcWidth;
    UINT32  u4ScrHeight;
    UINT32  u4SrcPitch;
    UINT32  u4ColorMode;
    UINT32  u4SrcClipX;
    UINT32  u4SrcClipY;
    UINT32  u4SrcClipWidth;
    UINT32  u4SrcClipHeight;
    UINT32  u4DisplayX;
    UINT32  u4DisplayY;
    UINT32  u4DisplayWidth;
    UINT32  u4DisplayHeight;
    UINT32  u4Index;
    UCHAR   ucFbgId;
    UCHAR   ucFbId;
    FBM_PIC_HDR_T *prFbmPicHdr;
    BOOL    fgEnable;

} SWVDEC_VDP_NFY_T;

typedef struct VDP_IMGRZ_INFO_T
{
    UINT32 u4ImgrzId;
    UINT32 u4Addr;
    UINT32 u4Size;
    BOOL   fgImgrzEnable;
    unsigned long long u8ImgrzAddCount;

}VDP_IMGRZ_INFO_T;

typedef struct VDP_IMGRZ_STATUS_T
{
    UINT32 u4ImgrzId;
    BOOL   fgReadyToShow;
}VDP_IMGERZ_STATUS_T;
static LIST_HEAD(rConsumeList);
static spinlock_t rConsumeLock;
static atomic_t rConsumeCount;
static wait_queue_head_t rConsumeQueue;
static BOOL fgMT53SimpleInit = FALSE;
static BOOL bImgrzSwitch = false;
static SWVDEC_VDP_CTX_T *prSwVdecVdpCtx = NULL;
static VDP_IMGRZ_INFO_T arVdpImgrzInfo[MAX_IMGRZINFO_NS] = {
        {
            .u4ImgrzId = -1,
            .u4Addr = 0,
            .u4Size = 0,
            .fgImgrzEnable = TRUE,
            .u8ImgrzAddCount = 0
        },
        {
            .u4ImgrzId = -1,
            .u4Addr = 0,
            .u4Size = 0,
            .fgImgrzEnable = TRUE,
            .u8ImgrzAddCount = 0
        }
};
static VDP_IMGERZ_STATUS_T arVdpImgrzStatus[MAX_IMGRZINFO_NS] = {
        {
            .u4ImgrzId = -1,
            .fgReadyToShow = FALSE
        },
        {
            .u4ImgrzId = -1,
            .fgReadyToShow = FALSE
        }
};
static SWVDEC_VDP_NFY_T rTmpNfyEvent;
static unsigned long long u8ImgrzAddCount = 0;

static UCHAR aucFbgId[MAX_CHANNEL_NS] = {FBM_FBG_ID_UNKNOWN, FBM_FBG_ID_UNKNOWN}; 
static UINT32 au4LastScrPitch[MAX_CHANNEL_NS] = {0, 0};
static UINT32 au4LastScrWidth[MAX_CHANNEL_NS] = {0, 0};
static UINT32 au4LastScrHeight[MAX_CHANNEL_NS] = {0, 0};

static UINT32 SwVdecVdp_GetImgrzIndex(UINT32 u4ImgrzID)
{
    UINT32 i;

    for(i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        if(arVdpImgrzInfo[i].u4ImgrzId == u4ImgrzID)
        {
            return i;
        }
    }

    return -1;
}
static UINT32 SwVdecVdp_ConfigImgrzID(UINT32 u4ImgrzIDMask, UINT32 *pImgrzCount)
{
    UINT32 au4ImgrzResizer[3] = {0};
    UINT32 u4ImgrzCount = 0;
    UINT32 u4ImgrzUsed = 0;

    if (u4ImgrzIDMask != 0)
    {
        if (u4ImgrzIDMask & IMGRZ_FIRST_RESIZER)
        {
            u4ImgrzCount++;
            au4ImgrzResizer[0] = 1;
        }
        if (u4ImgrzIDMask & IMGRZ_SECND_RESIZER)
        {
            u4ImgrzCount++;
            au4ImgrzResizer[1] = 2;
        }
        if (u4ImgrzIDMask & IMGRZ_THIRD_RESIZER)
        {
            u4ImgrzCount++;
            au4ImgrzResizer[2] = 3;
        }
    }
    if (u4ImgrzCount == 0)
    {
        u4ImgrzUsed = 0;
    }
    if (u4ImgrzCount == 1)
    {
        u4ImgrzUsed = au4ImgrzResizer[0]?au4ImgrzResizer[0]:(au4ImgrzResizer[1]?au4ImgrzResizer[1]:au4ImgrzResizer[2]);
    }
    else if (u4ImgrzCount >= 2) //if u4ImgrzCount >= 2, use 2 hardware imgrz.
    {
        u4ImgrzUsed = au4ImgrzResizer[0]?au4ImgrzResizer[0]:au4ImgrzResizer[1];
        if (bImgrzSwitch == TRUE)
        {
            if (u4ImgrzUsed == au4ImgrzResizer[0])
            {
                u4ImgrzUsed = au4ImgrzResizer[1]?au4ImgrzResizer[1]:au4ImgrzResizer[2];
            }
            else
            {
                u4ImgrzUsed = au4ImgrzResizer[2];
            }
            bImgrzSwitch = FALSE;
        }
        else
        {
            bImgrzSwitch = TRUE;
        }   
    }
    //Printf("%s(%d) u4ImgrzCount=%d, u4ImgrzIDMask=0x%x, u4ImgrzUsed=%d\n", __FUNCTION__, __LINE__, u4ImgrzCount, u4ImgrzIDMask, u4ImgrzUsed);

    *pImgrzCount = u4ImgrzCount;

    return u4ImgrzUsed;

}
static VOID SwVdecVdp_SetInfo(UINT32 u4ImgrzID, UINT32 u4ChannelID, UINT32 u4ScrAddr, UINT32 u4SrcWidth, UINT32 u4ScrHeight,
    UINT32 u4SrcPitch, UINT32 u4ColorMode, UINT32 u4SrcClipX, UINT32 u4SrcClipY, UINT32 u4SrcClipWidth, UINT32 u4SrcClipHeight,
	UINT32 u4DisplayX, UINT32 u4DisplayY, UINT32 u4DisplayWidth, UINT32 u4DisplayHeight, CHAR ucFbgId, 
	UCHAR ucFbId, FBM_PIC_HDR_T* prFbmPicHdr, SWVDEC_VDP_NFY_T *prNfyEvent)
{
    prNfyEvent->u4ImgrzID = u4ImgrzID;
    prNfyEvent->u4ChannelID = u4ChannelID;
    prNfyEvent->u4ScrAddr = u4ScrAddr;
    prNfyEvent->u4SrcWidth = u4SrcWidth;
    prNfyEvent->u4ScrHeight = u4ScrHeight;
    prNfyEvent->u4SrcPitch = u4SrcPitch;
    prNfyEvent->u4ColorMode = u4ColorMode;
    prNfyEvent->u4SrcClipX = u4SrcClipX;
    prNfyEvent->u4SrcClipY = u4SrcClipY;
    prNfyEvent->u4SrcClipWidth = u4SrcClipWidth;
    prNfyEvent->u4SrcClipHeight = u4SrcClipHeight;
    prNfyEvent->u4DisplayX = u4DisplayX;
    prNfyEvent->u4DisplayY = u4DisplayY;
    prNfyEvent->u4DisplayWidth = u4DisplayWidth;
    prNfyEvent->u4DisplayHeight = u4DisplayHeight;
    prNfyEvent->ucFbgId = ucFbgId;
    prNfyEvent->ucFbId = ucFbId;
    prNfyEvent->prFbmPicHdr = prFbmPicHdr;

}
UINT32 SwVdecVdp_Display(SWVDEC_VDP_NFY_T rNfyEvent)
{
    UINT32 u4PanelWidth, u4PanelHeight;
    VDP_REGION_T rRegion = {0};
    INT32 i = 0;

    au4LastScrPitch[rNfyEvent.u4ChannelID] = rNfyEvent.u4SrcPitch;
    au4LastScrWidth[rNfyEvent.u4ChannelID] = rNfyEvent.u4SrcWidth;
    au4LastScrHeight[rNfyEvent.u4ChannelID] = rNfyEvent.u4ScrHeight;        

    rNfyEvent.prFbmPicHdr->fgProgressiveFrm = TRUE;
    rNfyEvent.prFbmPicHdr->fgRepFirstFld = FALSE;
    rNfyEvent.prFbmPicHdr->fgTopFldFirst = FALSE;
    rNfyEvent.prFbmPicHdr->u4PTS = 1;

    //Display
    FBM_SetFrameBufferStatus(rNfyEvent.ucFbgId, rNfyEvent.ucFbId, (UCHAR)FBM_FB_STATUS_DISPLAYQ);

    // Unmute B2R
    VDP_SetMode(rNfyEvent.u4ChannelID, VDP_MODE_NORMAL);

    rRegion.u4X = rNfyEvent.u4SrcClipX * 10000 / rNfyEvent.u4SrcWidth;
    rRegion.u4Y = rNfyEvent.u4SrcClipY * 10000 / rNfyEvent.u4ScrHeight;
    rRegion.u4Width = rNfyEvent.u4SrcClipWidth * 10000 / rNfyEvent.u4SrcWidth;
    rRegion.u4Height = rNfyEvent.u4SrcClipHeight * 10000 / rNfyEvent.u4ScrHeight;
    VDP_SetSrcRegion(rNfyEvent.u4ChannelID, 0, rRegion);

    u4PanelWidth = (UINT32) 1920;//OSD_BASE_GetDisplayWidth();
    u4PanelHeight = (UINT32) 1080;//OSD_BASE_GetDisplayHeight();        

    rRegion.u4X = rNfyEvent.u4DisplayX * 10000 / u4PanelWidth;
    rRegion.u4Y = rNfyEvent.u4DisplayY * 10000 / u4PanelHeight;
    rRegion.u4Width = rNfyEvent.u4DisplayWidth * 10000 / u4PanelWidth;
    rRegion.u4Height = rNfyEvent.u4DisplayHeight * 10000 / u4PanelHeight;
    VDP_SetOutRegion(rNfyEvent.u4ChannelID, 0, rRegion);

    //Printf("%s: enter### %d %d \n", __FUNCTION__,rNfyEvent.u4Index,current->pid);
    VERIFY(x_sema_lock(prSwVdecVdpCtx->hImgrzSema, X_SEMA_OPTION_WAIT) == OSR_OK);
    arVdpImgrzInfo[rNfyEvent.u4Index].fgImgrzEnable = TRUE;
    arVdpImgrzInfo[rNfyEvent.u4Index].u4Addr = 0;
    arVdpImgrzInfo[rNfyEvent.u4Index].u4Size = 0;
    VERIFY(x_sema_unlock(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);
    for (i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        if(arVdpImgrzStatus[i].u4ImgrzId == rNfyEvent.u4ImgrzID)
        {
            arVdpImgrzStatus[i].fgReadyToShow = FALSE;
            break;
        }
    }
    //Printf("%s: end###  %d\n", __FUNCTION__, current->pid);

    return 0;

}
VOID SwVdecVdp_PutEvent(SWVDEC_VDP_CTX_T* prSwVdecVdp, SWVDEC_VDP_NFY_T* prNfyEvent)
{
    VERIFY(OSR_OK == x_msg_q_send(prSwVdecVdp->hEventMsgQ, (void *)(prNfyEvent), sizeof(SWVDEC_VDP_NFY_T), 255));
}

INT32 SwVdecVdp_GetEvent(SWVDEC_VDP_CTX_T* prSwVdecVdp)
{
    UINT16 u2MsgQIdx;
    SIZE_T zMsgSize = sizeof(SWVDEC_VDP_NFY_T);
    SWVDEC_VDP_NFY_T rNfyEvent;
    INT32 u4Index = 0;
    INT32 u4TmpIndex = 0;

    if(OSR_OK == x_msg_q_receive_timeout(&u2MsgQIdx, (void*)(&rNfyEvent), &zMsgSize, &(prSwVdecVdp->hEventMsgQ), 1, 100))
    {
        u4Index = SwVdecVdp_GetImgrzIndex(rNfyEvent.u4ImgrzID);
        if(u4Index < 0)
        {
            Printf("%s: can not find imgrz,id=%d\n", __FUNCTION__, rNfyEvent.u4ImgrzID);
            return -1;
        }
        rNfyEvent.u4Index = u4Index;
        u4TmpIndex = u4Index ? 0 : 1;
        if((arVdpImgrzInfo[u4TmpIndex].fgImgrzEnable == FALSE) && (arVdpImgrzInfo[u4TmpIndex].u8ImgrzAddCount < arVdpImgrzInfo[u4Index].u8ImgrzAddCount)
            && (arVdpImgrzInfo[u4TmpIndex].u8ImgrzAddCount != 0))
        {
            Printf("%s: first imgrz is working,should return\n", __FUNCTION__);
            memcpy(&rTmpNfyEvent, &rNfyEvent, sizeof(SWVDEC_VDP_NFY_T));
            rTmpNfyEvent.fgEnable = TRUE;
            return 0;
        }

        SwVdecVdp_Display(rNfyEvent);

        if(rTmpNfyEvent.fgEnable == TRUE)
        {
            SwVdecVdp_Display(rTmpNfyEvent);
            memset(&rTmpNfyEvent, 0 ,sizeof(SWVDEC_VDP_NFY_T));
        }
        return 0;
    }
    else
    {
        return -1;
    }

}

static INT32 SwVdecVdp_Imgrz_Done(UINT32 u4Which, IMGRZ_DONE_TYPE u4Type, VOID *pData)
{
    SWVDEC_VDP_NFY_T rNfyEvent;
    INT32 i = 0;

    //Printf("%s: enter###\n", __FUNCTION__);
    if(pData == NULL)
    {   
        Printf("%s: pData is null\n", __FUNCTION__);
        return -1;
    }
    memcpy(&rNfyEvent, (SWVDEC_VDP_NFY_T *)pData, sizeof(SWVDEC_VDP_NFY_T));
    x_mem_free(pData);
    pData = NULL;

    for (i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        if(arVdpImgrzStatus[i].u4ImgrzId == rNfyEvent.u4ImgrzID)
        {
            arVdpImgrzStatus[i].fgReadyToShow = TRUE;
            break;
        }
        else if(arVdpImgrzStatus[i].u4ImgrzId== -1)
        {
            arVdpImgrzStatus[i].u4ImgrzId = rNfyEvent.u4ImgrzID;
            arVdpImgrzStatus[i].fgReadyToShow = TRUE;
            break;
        }
    }
    SwVdecVdp_PutEvent(prSwVdecVdpCtx, &rNfyEvent);
    //Printf("%s: end###\n", __FUNCTION__);

    return 0;
    
}

static VOID ImgrzColorConvert_ProcessOne(VOID)
{
    while(prSwVdecVdpCtx->fgConvertThreadOneRunning)
    {
        SwVdecVdp_GetEvent(prSwVdecVdpCtx);
    }
    prSwVdecVdpCtx->hConvertThreadOne = (HANDLE_T)NULL;
    Printf("%s: ImgrzColorConvertProcessOne exit!\n", __FUNCTION__);
    x_thread_exit();
}
/**Start IMGRZ Color Convert Engine.
*@param  VOID               
*@retval  TRUE          Success.
*@retval  FALSE         Fail.
*/
static BOOL StartColorConvert_Engine(VOID)
{  
    Printf("%s: enter StartColorConvert_Engine\n", __FUNCTION__);
    prSwVdecVdpCtx->fgConvertThreadOneRunning = TRUE;
    if(x_thread_create(&prSwVdecVdpCtx->hConvertThreadOne,
            "ImgrzConvertThreadOne",
            2048, 40,
            (x_os_thread_main_fct)ImgrzColorConvert_ProcessOne, sizeof(void*), NULL) != OSR_OK)
    {
        Printf("%s: StartColorConvert_Engine thread create ERROR!\n", __FUNCTION__);
        return -1;
    }

    return 0;
}

/**Stop IMGRZ Color Convert Engine.
*@param  VOID               
*@retval  TRUE           Success.
*@retval  FALSE          Fail.
*/
static BOOL StopColorConvert_Engine(VOID)
{
    prSwVdecVdpCtx->fgConvertThreadOneRunning = FALSE;

    Printf("%s: enter StopColorConvert_Engine\n", __FUNCTION__);

    while (prSwVdecVdpCtx->hConvertThreadOne){
        x_thread_delay(10);
    }
    Printf("%s: StopColorConvert_Engine success\n", __FUNCTION__);

    return 0;
}

static inline CONSUME_BUFFER_T *ConsumeList_GetElement(SIMPLE_BUFFER_T *prSimBuf)
{
    CONSUME_BUFFER_T *pos;
	ASSERT((prSimBuf != NULL));

    list_for_each_entry(pos, &rConsumeList, link)
    {
        if(prSimBuf->u4Addr == pos->rBuffer.u4Addr && prSimBuf->u4Size == pos->rBuffer.u4Size)
        {
            return pos;
        }
    }

	return NULL;
}

static inline BOOL ComsumeList_AddElement(SIMPLE_BUFFER_T *prSimBuf)
{
    CONSUME_BUFFER_T *prConsumeBuf;	
    ASSERT((prSimBuf != NULL));

	prConsumeBuf = kmalloc(sizeof(CONSUME_BUFFER_T), GFP_KERNEL);
	prConsumeBuf->rBuffer = *prSimBuf;
	list_add_tail(&prConsumeBuf->link, &rConsumeList);
	atomic_add(1, &rConsumeCount);

	return TRUE;
}

static inline BOOL ComsumeList_RemoveElement(SIMPLE_BUFFER_T *prSimBuf)
{
    CONSUME_BUFFER_T *prConsumeBuf;	
    ASSERT((prSimBuf != NULL));

	if((prConsumeBuf = ConsumeList_GetElement(prSimBuf)) == NULL)
	{
	    return FALSE;
	}
	atomic_sub(1, &rConsumeCount);
	list_del(&prConsumeBuf->link);
	kfree(prConsumeBuf);
	
	return TRUE;
}

static inline BOOL ComsumeList_IsExistElement(SIMPLE_BUFFER_T *prSimBuf)
{
    ASSERT((prSimBuf != NULL));
	
    return ((ConsumeList_GetElement(prSimBuf) == NULL) ? FALSE : TRUE);
}

BOOL ComsumeList_HandleElement(SIMPLE_BUFFER_T *prSimBuf, ConsumeListHandle pfnHandle)
{
    BOOL ret;
	ASSERT((prSimBuf != NULL));
	ASSERT((pfnHandle != NULL));
	
    spin_lock(&rConsumeLock);
	ret = pfnHandle(prSimBuf);
	spin_unlock(&rConsumeLock);

	return ret;
}

VOID ComsumeList_WaitRemoveElement(SIMPLE_BUFFER_T *prSimBuf)
{
    INT32 i = 0;
    INT32 j = 0;

    ASSERT((prSimBuf != NULL));

    wait_event_interruptible(rConsumeQueue, 
		(ComsumeList_HandleElement(prSimBuf, ComsumeList_IsExistElement) == FALSE));
    for (i = 0; i < MAX_IMGRZINFO_NS; i++)
    {   
        while(arVdpImgrzInfo[i].u4Addr != 0
            && arVdpImgrzInfo[i].u4Addr == prSimBuf->u4Addr
            && arVdpImgrzInfo[i].u4Size == prSimBuf->u4Size
            && arVdpImgrzInfo[i].fgImgrzEnable != TRUE)
        {
            Printf("%s: imgrzid is busy,should wait addr=0x%x\n", __FUNCTION__, prSimBuf->u4Addr);
            x_thread_delay(2);
            j++;
            if(j >= 10)
            {
                break;
            }
        }
    }

}

VOID ComsumeList_WaitHasElement(VOID)
{
    wait_event_interruptible(rConsumeQueue,((atomic_read(&rConsumeCount) > 0)));
}

INT32 ConsumeList_Ioctl(UINT32 i4Cmd, UINT32 u4Arg)
{
    SIMPLE_BUFFER_T rSimBuf;
	
    if(i4Cmd == SIMPLE_ADD_SIMPLEBUFFER || i4Cmd == SIMPLE_REMOVE_SIMPLEBUFFER 
		|| i4Cmd == SIMPLE_WAIT_SIMPLEBUFFER_REMOVE)
	{
        if (copy_from_user(&rSimBuf, (void *)u4Arg, sizeof(SIMPLE_BUFFER_T)))
        {
            printk("%s(%d) - copy_from_user() fail\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }	
	}

    switch(i4Cmd)
    {
    case SIMPLE_ADD_SIMPLEBUFFER:
		{
			ComsumeList_HandleElement(&rSimBuf, ComsumeList_AddElement);
			wake_up_interruptible_all(&rConsumeQueue);
		}
	    break;
    case SIMPLE_REMOVE_SIMPLEBUFFER:
		{
			ComsumeList_HandleElement(&rSimBuf, ComsumeList_RemoveElement);
			wake_up_interruptible_all(&rConsumeQueue);
		}
	    break;
    case SIMPLE_WAIT_SIMPLEBUFFER_REMOVE:
		{
			ComsumeList_WaitRemoveElement(&rSimBuf);
		}
	    break;
	case SIMPLE_POLL_CONSUME:
		{
			ComsumeList_WaitHasElement();
		}
	    break;	
	default:
		{
			printk("%s(%d) - invalid cmd \n", __FUNCTION__, __LINE__);
		}
	    break;
    }	

	return 0;
}

static UCHAR VDPConvert_PathInit(UCHAR u4ChannelID, UINT32 u4Width, UINT32 u4Height, UINT32 u4Pitch)
{
    UCHAR ucFbgType = FBM_FBG_ID_UNKNOWN, ucFbgId = FBM_FB_ID_UNKNOWN;
    FBM_SEQ_HDR_T* prFbmSeqHdr = NULL;
    //VDP_REGION_T rRegion = {0};
    FBM_CREATE_FBG_PAR_T rPar;
    UINT8 u1VdecId = MAX_ES_NS;

    if(u4ChannelID == VDP_MAIN)
    {
        u1VdecId = NON_HW_ES_ID_MAIN;
    }
    else
    {
        u1VdecId = NON_HW_ES_ID_SUB;
    }

    x_memset(&rPar, 0, sizeof(FBM_CREATE_FBG_PAR_T));
    rPar.u1VdecId = u1VdecId;

	if(u4ChannelID == VDP_SUB)
	{
	    printk("call SRM_SetTvMode(SRM_TV_MODE_TYPE_PIP);");
	    SRM_SetTvMode(SRM_TV_MODE_TYPE_PIP);
	}
    ucFbgType = FBM_SelectGroupType(u4Width, u4Height);
    FBM_SetColorMode(FBM_CM_420);
    ucFbgId = FBM_CreateGroupExt(ucFbgType, FBM_VDEC_JPEG, u4Width, u4Height, &rPar);
    if(ucFbgId == FBM_FBG_ID_UNKNOWN)
    {
        printk("%s(%d)  Create Fbg Fail!! input resolution %dx%d\n", __FUNCTION__, __LINE__, u4Width, u4Height);
        return FBM_FBG_ID_UNKNOWN;
    }

    prFbmSeqHdr = FBM_GetFrameBufferSeqHdr(ucFbgId);
    if (!prFbmSeqHdr)
    {
        FBM_ReleaseGroup(ucFbgId);
        printk("%s(%d)  Create Fbg Fail!! input resolution %dx%d\n", __FUNCTION__, __LINE__, u4Width, u4Height);
        return FBM_FBG_ID_UNKNOWN;
    }
    
    prFbmSeqHdr->u2HSize = u4Width;
    prFbmSeqHdr->u2VSize = u4Height;
    prFbmSeqHdr->u2OrgHSize = prFbmSeqHdr->u2HSize;
    prFbmSeqHdr->u2OrgVSize = prFbmSeqHdr->u2VSize;
    prFbmSeqHdr->u2LineSize = u4Pitch ? (((u4Pitch + 15) >> 4) << 4) : (((u4Width + 15) >> 4) << 4);
    prFbmSeqHdr->fgProgressiveSeq = TRUE;
    prFbmSeqHdr->fgRasterOrder = TRUE;
    prFbmSeqHdr->fg422Mode = FALSE;
    prFbmSeqHdr->ucFrmRatCod = MPEG_FRAME_RATE_30;

    bApiVideoSetVideoSrc(u4ChannelID, u4ChannelID ? SV_VS_DT2 : SV_VS_DT1);
    VDP_SetEnable(u4ChannelID, 1);
    VDP_SetMode(u4ChannelID, VDP_MODE_BG);
    VDP_SetInput(u4ChannelID, u1VdecId, 0);
    //VDP_SetInput(u4ChannelID, 0, 0);
    FBM_SetPlayMode(ucFbgId, FBM_FBG_MM_MODE);
    FBM_FbgChgNotify(ucFbgId, u1VdecId);

    FBM_SetFrameBufferFlag(ucFbgId, FBM_FLAG_SEQ_CHG_SPEEDUP);

    return ucFbgId;
}


static INT32 VDPConvert_PathRelease(UCHAR u4ChannelID)
{
	VDP_REGION_T rRegion = {0};
    INT32 i = 0;
    INT32 j = 0;

    while (arVdpImgrzInfo[0].fgImgrzEnable != TRUE || arVdpImgrzInfo[1].fgImgrzEnable != TRUE)
    {
        x_thread_delay(4);
        j++;
        if(j >= 10)
        {
            break;
        }
        Printf("%s: imgrz is working,imgrzEnable=%d,%d,%d\n", __FUNCTION__, arVdpImgrzInfo[0].fgImgrzEnable, arVdpImgrzInfo[1].fgImgrzEnable, current->pid);
    }
    VDP_SetEnable(u4ChannelID, 0);
	bApiVideoSetVideoSrc(u4ChannelID, SV_VS_MAX);

    if (aucFbgId[u4ChannelID] != FBM_FBG_ID_UNKNOWN)
    {
        FBM_FbgChgNotify(FBM_FBG_ID_UNKNOWN, 0);
        FBM_ReleaseGroup(aucFbgId[u4ChannelID]);
        aucFbgId[u4ChannelID] = FBM_FBG_ID_UNKNOWN;
		if(u4ChannelID == VDP_SUB)
		{
		    printk("call SRM_SetTvMode(SRM_TV_MODE_TYPE_NORMAL);");
			SRM_SetTvMode(SRM_TV_MODE_TYPE_NORMAL);
		}
    }
	//stop convert should set out region is 0,if not set sub will cover main.
	VDP_SetOutRegion(u4ChannelID, 0, rRegion);
	

    VERIFY(x_sema_lock(prSwVdecVdpCtx->hImgrzSema, X_SEMA_OPTION_WAIT) == OSR_OK);
    for(i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        arVdpImgrzInfo[i].u4ImgrzId = -1;
        arVdpImgrzInfo[i].fgImgrzEnable = TRUE;
        arVdpImgrzInfo[1].u4Addr = 0;
        arVdpImgrzInfo[1].u4Size = 0;
        arVdpImgrzInfo[i].u8ImgrzAddCount = 0;
    }
    VERIFY(x_sema_unlock(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);

    return 0;
}

static VOID VDPConvert_ColorCvt(UINT32 u4ImgrzUsed, UINT32 u4SrcAddr, UINT32 u4Width, UINT32 u4Height, 
        UINT32 u4Pitch, UINT32 u4ColorMode, UINT32 u4DstYAddr, UINT32 u4DstCAddr, SWVDEC_VDP_NFY_T *prNfyEvent)
{
    RZ_JPG_SCL_PARAM_SET_T rJpgParam;

    //from YV12 to yc420

    //Printf("%s: enter###\n", __FUNCTION__);
    x_memset((void *)&rJpgParam, 0, sizeof(RZ_OSD_SCL_PARAM_SET_T));

    rJpgParam.u4IsRsIn	  = 1; //block/raster in
    rJpgParam.u4YSrcBase1	= (u4SrcAddr);
    rJpgParam.u4YSrcBase2	= (u4SrcAddr);

    rJpgParam.u4YSrcBufLen	= u4Pitch;
    rJpgParam.u4CSrcBufLen	= u4Pitch >> 1;

    rJpgParam.u4YSrcHOffset = 0;
    rJpgParam.u4YSrcVOffset = 0;
    rJpgParam.u4YSrcW	  = u4Width;
    rJpgParam.u4YSrcH	  = u4Height;

    rJpgParam.u4CbSrcBase1	= (u4SrcAddr + u4Pitch*u4Height + u4Pitch*u4Height/4);
    rJpgParam.u4CbSrcBase2	= (u4SrcAddr + u4Pitch*u4Height + u4Pitch*u4Height/4);
    rJpgParam.u4CbSrcHOffset	  = 0;
    rJpgParam.u4CbSrcVOffset	  = 0;
    rJpgParam.u4CbSrcW	  = u4Width >> 1;
    rJpgParam.u4CbSrcH	  = u4Height >> 1;

    rJpgParam.u4CrSrcBase1	= (u4SrcAddr + u4Pitch*u4Height);
    rJpgParam.u4CrSrcBase2	= (u4SrcAddr + u4Pitch*u4Height);
    rJpgParam.u4CrSrcHOffset = 0;
    rJpgParam.u4CrSrcVOffset = 0;
    rJpgParam.u4CrSrcW = u4Width >> 1;
    rJpgParam.u4CrSrcH = u4Height >> 1;

    rJpgParam.u4IsRsOut = 1;	//block/raster out
    rJpgParam.u4OutSwap = 0;
    rJpgParam.u4ColorComp = 7; //y,cb,cr exists

    rJpgParam.u4YTgBase   = u4DstYAddr;
    rJpgParam.u4CTgBase   = u4DstCAddr;

    rJpgParam.u4IsJpgSameaddr = 0;
    rJpgParam.u4YTgHOffset = 0;
    rJpgParam.u4YTgVOffset = 0;
    rJpgParam.u4YTgBufLen = u4Pitch;
    rJpgParam.u4CTgBufLen = u4Pitch;

    rJpgParam.u4IsVdo2Osd = 0;	   // output in OSD format ?
    rJpgParam.u4YTgCM = 0;
    rJpgParam.u4YTgW = u4Width;
    rJpgParam.u4YTgH = u4Height;
    rJpgParam.u4OutMode = E_RZ_VDO_OUTMD_420;
    rJpgParam.u4CTgW = u4Width >> 1;
    rJpgParam.u4CTgH = u4Height >> 1;

    rJpgParam.u4IsFstBl = 1;
    rJpgParam.u4IsLstBl = 1;
    rJpgParam.u4JpgVfacY = 1;
    rJpgParam.u4JpgVfacCb = 0;
    rJpgParam.u4JpgVfacCr = 0;
    rJpgParam.fgBlassign = TRUE; //indicates to directly assign input/output address

    IMGRZ_Lock_Ex(u4ImgrzUsed);
    IMGRZ_ReInit_Ex(u4ImgrzUsed);
    rJpgParam.u4RzIOMMU_MMUTableAddr = HalCpuGetPhyPgd();
    IMGRZ_Cfg_MMU_IO_Ex(u4ImgrzUsed, (BOOL) 1, (BOOL) 0);
    IMGRZ_SetScaleOpt_Ex(u4ImgrzUsed, E_RZ_INOUT_TYPE_JPGMD);
    IMGRZ_Async_Ex(u4ImgrzUsed, SwVdecVdp_Imgrz_Done, (void *)prNfyEvent);
    IMGRZ_Scale_Ex(u4ImgrzUsed, &rJpgParam);
    IMGRZ_Flush_Ex(u4ImgrzUsed);
    IMGRZ_Wait_Ex(u4ImgrzUsed);
    IMGRZ_Unlock_Ex(u4ImgrzUsed);
    //Printf("%s: end###\n", __FUNCTION__);


    return ;
}

#include <linux/file_ops.h>
static int SwVdecVdp_DumpRawdata(UINT32 u4SrcAddr, UINT32 u4SrcPitch, UINT32 u4SrcWidth, UINT32 u4SrcHeight)
{
    UINT32 dumpBuf = GFX_GetTmpBuf();
    struct file *file;
    void *rawData = NULL;
    UINT32 size = 0;
    UINT64 offset = 0;
    const char *filePath = "/data/swvdec_rawdata.bin";

    GFX_Lock();
    GFX_MMU_Setting(TRUE, TRUE, 0);
    //don't care about the color mode here, u4SrcPitch is the length of Y
    GFX_SetSrc((UINT8 *)u4SrcAddr, CM_ARGB4444_DIRECT16, u4SrcPitch * 3 / 2);
    GFX_SetDst((UINT8 *)dumpBuf, CM_ARGB4444_DIRECT16, u4SrcPitch * 3 / 2);
    GFX_SetBltOpt((UINT32)D_GFXFLAG_NONE, 0x00000000, 0xffffffff);
    GFX_BitBlt(0, 0, 0, 0, u4SrcWidth, u4SrcHeight);
    GFX_Flush();
    GFX_Task_Wait_Sync();
    GFX_MMU_Set_Enable(FALSE);
    GFX_Unlock();

    file = FILE_Open(filePath, O_WRONLY|O_CREAT, 0644);

    if (!file)
    {
        printk("open %s failed\n", filePath);
        return -1;
    }
    size = u4SrcPitch * u4SrcHeight * 3 / 2;
    rawData = ioremap(dumpBuf, size);
    size = FILE_Write(file, (unsigned char *)rawData, size, &offset);
    printk("written size:%ld\n", size);
    iounmap(rawData);
    FILE_Close(file);
    return 0;
}

static BOOL VDPConvert_FBFlip(UINT32 u4ChannelID, UINT32 u4ImgrzIDMask, UINT32 u4ScrAddr, UINT32 u4SrcSize,
        UINT32 u4SrcWidth, UINT32 u4ScrHeight, UINT32 u4SrcPitch, UINT32 u4ColorMode,
	    UINT32 u4SrcClipX, UINT32 u4SrcClipY, UINT32 u4SrcClipWidth, UINT32 u4SrcClipHeight,
	    UINT32 u4DisplayX, UINT32 u4DisplayY, UINT32 u4DisplayWidth, UINT32 u4DisplayHeight)
{
    UCHAR ucFbgId;
    UCHAR ucFbId;
    INT32 i;
    INT32 i4Count = 0;
    UINT32 u4ImgrzID;
    UINT32 u4ImgrzCount;
    UINT32 u4DstYAddr, u4DstCAddr;
    FBM_PIC_HDR_T *prFbmPicHdr = NULL;
    static UINT32 debugValue = 0;
    //UINT32 u4PanelWidth, u4PanelHeight;
    //VDP_REGION_T rRegion = {0};
    SWVDEC_VDP_NFY_T *prNfyEvent = NULL;
    //MTIOMMU_PIN_RANGE_T rPinRange = {0};
    //unsigned long u4Arg = 0;

    ASSERT((u4SrcPitch % 32 == 0));
    ASSERT((u4SrcPitch != 0));

    if(u4ChannelID >= MAX_CHANNEL_NS)
    {
        return FALSE;
    }

    if (debugValue != OSD_Get_Dbg_Info(14))
    {
        debugValue = OSD_Get_Dbg_Info(14);
        SwVdecVdp_DumpRawdata(u4ScrAddr, u4SrcPitch, u4SrcWidth, u4ScrHeight);
    }
    u4ImgrzID = SwVdecVdp_ConfigImgrzID(u4ImgrzIDMask, &u4ImgrzCount);

    for (i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        if(arVdpImgrzInfo[i].u4ImgrzId == u4ImgrzID)
        {
            while(arVdpImgrzInfo[i].fgImgrzEnable == FALSE)
            {
                x_thread_delay(1);
                i4Count++;
                if(i4Count >= 10)
                {
                    if((i4Count >= 100)
                            || (arVdpImgrzStatus[0].u4ImgrzId == u4ImgrzID && arVdpImgrzStatus[0].fgReadyToShow == TRUE)
                            || (arVdpImgrzStatus[1].u4ImgrzId == u4ImgrzID && arVdpImgrzStatus[1].fgReadyToShow == TRUE))
                    {
                        VERIFY(x_sema_lock(prSwVdecVdpCtx->hImgrzSema, X_SEMA_OPTION_WAIT) == OSR_OK);
                        arVdpImgrzInfo[i].fgImgrzEnable = TRUE;
                        arVdpImgrzInfo[i].u4Addr = 0;
                        arVdpImgrzInfo[i].u4Size = 0;
                        VERIFY(x_sema_unlock(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);
                        Printf("%s[%d]: i4Count = %d, imgrzid = %d is busy,should wait\n", __FUNCTION__, __LINE__, i4Count, u4ImgrzID);
                        i4Count = 0;
                        break;
                    }
                }
                //Printf("%s: imgrzid = %d is busy,should wait\n", __FUNCTION__, u4ImgrzID);
            }
        }
    }

    prNfyEvent = (SWVDEC_VDP_NFY_T *) x_mem_calloc(1, sizeof(SWVDEC_VDP_NFY_T));
    if(prNfyEvent == NULL)
    {
        Printf("%s: SWVDEC_VDP_NFY_T malloc fail\n", __FUNCTION__);
        return FALSE;
    }
    memset(prNfyEvent, 0 ,sizeof(SWVDEC_VDP_NFY_T));

    if (aucFbgId[u4ChannelID] == FBM_FBG_ID_UNKNOWN)
    {
        aucFbgId[u4ChannelID] = VDPConvert_PathInit(u4ChannelID, u4SrcWidth, u4ScrHeight, u4SrcPitch);
    }
    else if ((au4LastScrPitch[u4ChannelID] != u4SrcPitch) ||
            (au4LastScrWidth[u4ChannelID] != u4SrcWidth) ||
            (au4LastScrHeight[u4ChannelID] != u4ScrHeight))
    {
        VDPConvert_PathRelease(u4ChannelID);
        aucFbgId[u4ChannelID] = VDPConvert_PathInit(u4ChannelID, u4SrcWidth, u4ScrHeight, u4SrcPitch);
    }

    ucFbgId = aucFbgId[u4ChannelID];

    if (ucFbgId == (UCHAR)FBM_FB_ID_UNKNOWN)
    {
        Printf("%s(%d) ucFbgId Create Fail.\n", __FUNCTION__, __LINE__);
        VDPConvert_PathRelease(u4ChannelID);
        x_mem_free(prNfyEvent);
        prNfyEvent = NULL;
        return FALSE;
    }

    ucFbId = FBM_GetEmptyFrameBuffer(ucFbgId, (UINT32)500);
    if (ucFbId == (UCHAR)FBM_FB_ID_UNKNOWN)
    {
        //Enhance for: too many log issue.
        static INT32 count = 0;
        count++;
        if (count<10 || 0 == count%10)
        {
            Printf("%s(%d) Get empty frame buffer failed.\n", __FUNCTION__, __LINE__);
        }

        if (count >= 100)
        {
            count = 0;
        }
        x_mem_free(prNfyEvent);
        prNfyEvent = NULL;
        return FALSE;
    }

    FBM_GetFrameBufferAddr(ucFbgId, ucFbId, &u4DstYAddr, &u4DstCAddr);
    prFbmPicHdr = FBM_GetFrameBufferPicHdr(ucFbgId, ucFbId);
    if ((prFbmPicHdr == NULL) || (u4DstYAddr == 0) || (u4DstCAddr == 0))
    {
        Printf("%s(%d) Alloc frame buffer failed.\n", __FUNCTION__, __LINE__);
        x_mem_free(prNfyEvent);
        prNfyEvent = NULL;
        return FALSE;
    }

    for (i = 0; i < MAX_IMGRZINFO_NS; i++)
    {
        if(arVdpImgrzInfo[i].u4ImgrzId == u4ImgrzID)
        {
            //Printf("%s: enter 111111111111111 i=%d,u4ImgrzID =%d,addr=0x%x\n", __FUNCTION__,i,u4ImgrzID,u4ScrAddr);
            if(arVdpImgrzInfo[i].fgImgrzEnable == TRUE)
            {
                VERIFY(x_sema_lock(prSwVdecVdpCtx->hImgrzSema, X_SEMA_OPTION_WAIT) == OSR_OK);
                arVdpImgrzInfo[i].fgImgrzEnable = FALSE;
                arVdpImgrzInfo[i].u4Addr = u4ScrAddr;
                arVdpImgrzInfo[i].u4Size = u4SrcSize;
                arVdpImgrzInfo[i].u8ImgrzAddCount = ++u8ImgrzAddCount;
                VERIFY(x_sema_unlock(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);
            }
            break;
        }
        else if(arVdpImgrzInfo[i].u4ImgrzId == -1)
        {
            //Printf("%s: enter 2222222222222 i=%d,u4ImgrzID=%d,addr=0x%x\n", __FUNCTION__, i,u4ImgrzID, u4ScrAddr);
            VERIFY(x_sema_lock(prSwVdecVdpCtx->hImgrzSema, X_SEMA_OPTION_WAIT) == OSR_OK);
            arVdpImgrzInfo[i].u4ImgrzId = u4ImgrzID;
            arVdpImgrzInfo[i].fgImgrzEnable = FALSE;
            arVdpImgrzInfo[i].u4Addr = u4ScrAddr;
            arVdpImgrzInfo[i].u4Size = u4SrcSize;
            arVdpImgrzInfo[i].u8ImgrzAddCount = ++u8ImgrzAddCount;
            VERIFY(x_sema_unlock(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);
            break;
        }
    }

    SwVdecVdp_SetInfo(u4ImgrzID, u4ChannelID, u4ScrAddr, u4SrcWidth, u4ScrHeight, u4SrcPitch, u4ColorMode,
            u4SrcClipX, u4SrcClipY, u4SrcClipWidth, u4SrcClipHeight, u4DisplayX, u4DisplayY, u4DisplayWidth, u4DisplayHeight,
            ucFbgId, ucFbId, prFbmPicHdr, prNfyEvent);
    //call xinwei supported function to color space convert src buffer to b2r buffer u4DstYAddr, u4DstCAddr 
    VDPConvert_ColorCvt(u4ImgrzID, u4ScrAddr, u4SrcWidth, u4ScrHeight, u4SrcPitch, u4ColorMode, u4DstYAddr, u4DstCAddr, prNfyEvent);

    return TRUE;
}


static INT32 VDPConvert_FlipToB2R(VDPC_INFO_T *prInfo)
{
    UINT32 u4BmpAddr, u4BmpSize, u4BmpWidth, u4BmpHeight, u4BmpPitch, u4ColorMode;
    UINT32 u4SrcClipX, u4SrcClipY, u4SrcClipWidth, u4SrcClipHeight;
    UINT32 u4DisplayX, u4DisplayY, u4DisplayWidth, u4DisplayHeight;
    UINT32 u4ChannelID, u4ImgrzIDMask;
    BOOL  fgRet = TRUE;

    ASSERT((prInfo != NULL));
    if (!prInfo || (prInfo->u4ChannelID >= MAX_CHANNEL_NS))
    {
        printk("%s(%d) - bad argument \n", __FUNCTION__, __LINE__);
        return -1;
    }

    u4BmpAddr = prInfo->rSrcBuffer.rBuffer.u4Addr;
    u4BmpSize = prInfo->rSrcBuffer.rBuffer.u4Size;
    u4BmpWidth = prInfo->rSrcBuffer.u4Width;
    u4BmpHeight = prInfo->rSrcBuffer.u4Height;
    u4BmpPitch = prInfo->rSrcBuffer.u4Pitch;
    u4ColorMode = prInfo->rSrcBuffer.u4ColorMode;

    u4SrcClipX = prInfo->u4SrcClipX;
    u4SrcClipY = prInfo->u4SrcClipY;
    u4SrcClipWidth = prInfo->u4SrcClipWidth;
    u4SrcClipHeight = prInfo->u4SrcClipHeight;

    u4DisplayX = prInfo->u4DisplayX;
    u4DisplayY = prInfo->u4DisplayY;
    u4DisplayWidth = prInfo->u4DisplayWidth;
    u4DisplayHeight = prInfo->u4DisplayHeight;

    u4ChannelID = prInfo->u4ChannelID;
    //hwcomposer set imgrzID has the mask.
    u4ImgrzIDMask = prInfo->u4ImgrzID;

    if((u4SrcClipX + u4SrcClipWidth) > u4BmpWidth || (u4SrcClipY + u4SrcClipHeight) > u4BmpHeight)
    {
        printk("%s(%d) - bad argument \n", __FUNCTION__, __LINE__);
        return -1;
    }

    fgRet = VDPConvert_FBFlip(u4ChannelID, u4ImgrzIDMask, u4BmpAddr, u4BmpSize, u4BmpWidth, u4BmpHeight, u4BmpPitch,
        u4ColorMode, u4SrcClipX, u4SrcClipY, u4SrcClipWidth, u4SrcClipHeight, u4DisplayX, u4DisplayY, u4DisplayWidth, u4DisplayHeight);

    return fgRet ? 0 : -1;
}

INT32 VDPConvert_Ioctl(UINT32 i4Cmd, UINT32 u4Arg)
{
    VDPC_INFO_T rInfo;
    if(i4Cmd == SIMPLE_VDP_CONVERT_INIT)
    {
        if(prSwVdecVdpCtx == NULL)
        {
            prSwVdecVdpCtx = (SWVDEC_VDP_CTX_T *) x_mem_calloc(1, sizeof(SWVDEC_VDP_CTX_T));
            if(prSwVdecVdpCtx == NULL)
            {
                Printf("SWVDEC_VDP_CTX_T malloc fail\n");
                return -1;
            }
            memset(prSwVdecVdpCtx, 0, sizeof(SWVDEC_VDP_CTX_T));
        }

        if(prSwVdecVdpCtx->hEventMsgQ == NULL_HANDLE)
        {
            CHAR cBuf[64];
            x_snprintf(cBuf, sizeof(cBuf), "SWVDEC_VDP_EVENTQ%x", prSwVdecVdpCtx);
            VERIFY(OSR_OK == x_msg_q_create(&prSwVdecVdpCtx->hEventMsgQ, cBuf, sizeof(SWVDEC_VDP_NFY_T), IMGRZ_CMD_QUEUE_SIZE));
            VERIFY(OSR_OK == x_sema_create(&prSwVdecVdpCtx->hImgrzSema, X_SEMA_TYPE_MUTEX, X_SEMA_STATE_UNLOCK));
        }

        StartColorConvert_Engine();
        memset((void *)&rTmpNfyEvent, 0, sizeof(SWVDEC_VDP_NFY_T));
    }
    else if(i4Cmd == SIMPLE_VDP_CONVERT_DEINIT)
    {
        if(prSwVdecVdpCtx != NULL)
        {
            StopColorConvert_Engine();

            if(prSwVdecVdpCtx->hEventMsgQ != NULL_HANDLE)
            {
                VERIFY(x_msg_q_delete(prSwVdecVdpCtx->hEventMsgQ) == OSR_OK);
                VERIFY(x_sema_delete(prSwVdecVdpCtx->hImgrzSema) == OSR_OK);
                prSwVdecVdpCtx->hEventMsgQ = NULL_HANDLE;
                prSwVdecVdpCtx->hImgrzSema = NULL_HANDLE;
            }

            x_mem_free(prSwVdecVdpCtx);
            prSwVdecVdpCtx = NULL;
        }
    }
    else if(i4Cmd == SIMPLE_VDP_PIN_MEM)
    {
    //    MTIOMMU_PIN_RANGE_T rPinRange;
       // if (copy_from_user(&rPinRange, (void *)u4Arg, sizeof(MTIOMMU_PIN_RANGE_T)))
       // {
         //   printk("%s(%d) - copy_from_user() fail\n", __FUNCTION__, __LINE__);
           // return -EFAULT;
       // }
       // HalFlushInvalidateDCacheMultipleLine(rPinRange.start, rPinRange.size);
        return _MTMMU_IoMap_PinMem(u4Arg);
    }

    if(i4Cmd == SIMPLE_VDP_CONVERT || i4Cmd == SIMPLE_VDP_STOP_CVT)
    {
        if (copy_from_user(&rInfo, (void *)u4Arg, sizeof(VDPC_INFO_T)))
        {
            printk("%s(%d) - copy_from_user() fail\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }	

        if(i4Cmd == SIMPLE_VDP_CONVERT)
        {
            return VDPConvert_FlipToB2R(&rInfo);
        }
        else if(i4Cmd == SIMPLE_VDP_STOP_CVT)
        {
            return VDPConvert_PathRelease(rInfo.u4ChannelID);
        }
    }

    return 0;
}

INT32 PanelCmd_Ioctl(UINT32 i4Cmd, UINT32 u4Arg)
{
#ifdef CC_QUIET_HOT_BOOT_SUPPORT
	//HWC should not turn on panel due to QHB
	DTVCFG_T rDtvCfg;
#endif

	if(i4Cmd == SIMPLE_PANEL_POWER)
	{
#ifdef CC_QUIET_HOT_BOOT_SUPPORT
		EEPDTV_GetCfg(&rDtvCfg);
		if(rDtvCfg.u1Flags4 & DTVCFG_FLAG4_QUIET_HOT_BOOT)
		{
			printk("%s(%d) HWC should not turn on panel due to QHB\n", __FUNCTION__, __LINE__);
		}
		else
#endif
		{
			vApiPanelPowerSequenceEx((BOOL)u4Arg);
			printk("%s(%d) vApiPanelPowerSequenceEx u4Arg=%d\n", __FUNCTION__, __LINE__, u4Arg);
		}
	}

	return 0;
}

INT32 HdrCmd_Ioctl(UINT32 i4Cmd, UINT32 u4Arg)
{
    HDR_CAPABILITIES_T rHdrInfo;
    INT32 i4Ret = 0;

    if (i4Cmd == SIMPLE_HDR_GET_CAPABILITIES)
    {
        if ((void *)u4Arg == NULL)
        {
            printk("%s(%d) - u4Arg is NULL\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }
        if (copy_from_user(&rHdrInfo, (void *)u4Arg, sizeof(HDR_CAPABILITIES_T)))
        {
            printk("%s(%d) - copy_from_user() fail\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }
        i4Ret = vDrvGetHDRCapabilities(&rHdrInfo.u4NumTypes, rHdrInfo.u4Types, &rHdrInfo.u4MaxLuminance,
                       &rHdrInfo.u4MaxAverageLuminance, &rHdrInfo.u4MinLuminance);
        if (i4Ret < 0)
        {
            printk("%s(%d) - vDrvGetHDRCapabilities() fail\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }
        printk("%s(%d) - u4NumTypes=%d,u4Types=%p,u4MaxLuminance=%d,u4MaxAverageLuminance=%d,u4MinLuminance=%d\n",
                __FUNCTION__, __LINE__, rHdrInfo.u4NumTypes, rHdrInfo.u4Types, rHdrInfo.u4MaxLuminance,
                rHdrInfo.u4MaxAverageLuminance, rHdrInfo.u4MinLuminance);
        if (copy_to_user((void *)u4Arg, &rHdrInfo, sizeof(HDR_CAPABILITIES_T)))
        {
            printk("%s(%d) - copy_to_user() fail\n", __FUNCTION__, __LINE__);
            return -EFAULT;
        }
    }

    return 0;
}

/*
static int MT53Simple_Open(struct inode *inode, struct file *file)
{
    UN_USED(inode);
	UN_USED(file);
	printk("MT53Simple_Open  .....\n");
	return 0;
}

static int MT53Simple_Release(struct inode *inode, struct file *file)
{
    UN_USED(inode);
	UN_USED(file);
	printk("MT53Simple_Release  .....\n");
	return 0;
}


static long MT53Simple_Ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
	UN_USED(file);

	printk("MT53Simple_Ioctl test .....\n");

	if(cmd >= SIMPLE_CONSUME_CMD_START && cmd <= SIMPLE_CONSUME_CMD_END)
	{
	    return ConsumeList_Ioctl(cmd, arg);
	}

	if(cmd >= SIMPLE_VDP_CMD_START && cmd <= SIMPLE_VDP_CMD_END)
	{
        return VDPConvert_Ioctl(cmd, arg); 
	}

	if(cmd == SIMPLE_TEST_IO)
	{
	    printk("MT53Simple_Ioctl test \n");
	}

	return 0;
}

static struct file_operations rMT53SimpleOps = {
    .owner = THIS_MODULE,
	.open = MT53Simple_Open,
	.release = MT53Simple_Release,
	.unlocked_ioctl = MT53Simple_Ioctl
};

static struct miscdevice rMiscSimpleDev = {
    .minor = MISC_DYNAMIC_MINOR,
	.name = "simple",
	.fops = &rMT53SimpleOps,
	.mode = S_IRUGO | S_IWUGO
};

VOID Mt53Simple_InitDevice(VOID)
{
    INT32 ret;

	printk("call Mt53Simple_InitDevice !!! \n");

    if(TRUE == fgMT53SimpleInit)
    {
        return;
    }

	fgMT53SimpleInit = TRUE;
	
	spin_lock_init((&rConsumeLock));
    atomic_set(&rConsumeCount, 0);
    init_waitqueue_head(&rConsumeQueue);


	ret = misc_register(&rMiscSimpleDev);
	if(ret < 0)
	{
	    printk("unable register simple device !!! \n");
	}
	
}
*/


INT32 mt53simple_init(void)
{
    if(TRUE == fgMT53SimpleInit)
    {
        return 0;
    }

	fgMT53SimpleInit = TRUE;
	
	spin_lock_init((&rConsumeLock));
    atomic_set(&rConsumeCount, 0);
    init_waitqueue_head(&rConsumeQueue);

	return 0;
}

INT32 mt53simple_ioctl(UINT32 u4Arg)
{
	SIMPLE_CMD_T rCmd;
	
	if (copy_from_user(&rCmd, (void *)u4Arg, sizeof(SIMPLE_CMD_T)))
	{
		printk("%s(%d) - copy_from_user() fail\n", __FUNCTION__, __LINE__);
		return -EFAULT;
	}

	if(rCmd.u4Cmd >= SIMPLE_CONSUME_CMD_START && rCmd.u4Cmd <= SIMPLE_CONSUME_CMD_END)
	{
	    return ConsumeList_Ioctl(rCmd.u4Cmd, rCmd.u4Arg);
	}

	if(rCmd.u4Cmd >= SIMPLE_VDP_CMD_START && rCmd.u4Cmd <= SIMPLE_VDP_CMD_END)
	{
        return VDPConvert_Ioctl(rCmd.u4Cmd, rCmd.u4Arg); 
	}

	if(rCmd.u4Cmd >= SIMPLE_PANEL_CMD_START && rCmd.u4Cmd <= SIMPLE_PANEL_CMD_END)
	{
		return PanelCmd_Ioctl(rCmd.u4Cmd, rCmd.u4Arg);
	}

	if (rCmd.u4Cmd >= SIMPLE_HDR_CMD_START && rCmd.u4Cmd <= SIMPLE_HDR_CMD_END)
    {
        return HdrCmd_Ioctl(rCmd.u4Cmd, rCmd.u4Arg);
    }

	if(rCmd.u4Cmd == SIMPLE_TEST_IO)
	{
	    printk("mt53simple_ioctl test \n");
	}

	return 0;
}

#endif

